############################################################################
#Name: ArmaturePhysicsGui
#Author: Rickard Jonsson
#Version: Blender 2.49.2
#
#This module is part of the ArmaturePhysics library, a set of modules
#that simulate physical motion of armature objects, and armature bones.
############################################################################ 
#
#This module contains the functions for creating the graphical user interface
#of the ArmaturePhysics modules. 
############################################################################
############################################################################
""" 
ArmaturePhysics library
Copyright (c) <2010> <Rickard Jonsson>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
"""
############################################################################
############################################################################


import Blender
from Blender import Draw			#For the graphical interface
from Blender.Mathutils import *		#For Vectors and matrices
import ArmaturePhysics
reload(ArmaturePhysics)				#For the development phase


def InitGui(FrameStartEnd,AName,InitialParameterDict,MainPDict,MonitorPDict):
	#Initializing parameters and button settings for the graphical user interface.
	#
	#The first time the script is run in the Blender session in question,
	#the parameters in the dictionar MainPDict is used to set the button toggle values etc.
	#If one exits the script and starts it up again (without quitting Blender), 
	#the settings (toggle button clicks etc) made in the user interface will be remembered,
	#because they are stored in the ArmaturePhysics module (which is still up and running despite exiting the GUI),
	#so when one starts up the GUI a second time the values are read from ArmaturePhysics.GUI_DICT (and .GUI_start, .GUI_span and .GUI_back).
	#
	#Note it is quite possible that there is some inbuilt way of achieving this "memory" that I am unaware of.
	#
	#Improvement: Really the .GUI_start, .GUI_span and .GUI_back should be included in the GUI_DICT.
	
	global start,span,back
	global grav,object,dyn,boneforce,steps,dynderivative,ndyn,nmax,dampmode,dynfast
	global ArmatureName,MainParameterDict,MonitorParameterDict
	global almostrigid, updatealmostrigid
	
	global PhysOb
	
	##########
	
	ArmatureName=AName
	MainParameterDict=MainPDict
	MonitorParameterDict=MonitorPDict
	
	PhysOb=ArmaturePhysics.InitializeBoneStructure(InitialParameterDict)
			
	########## Reading or writing to a parameter dictionary stored in a 'live' ArmaturePhysics module session.
	
	try:
		dummy=ArmaturePhysics.GUI_DICT  	#Checking if previous GUI settings exist 
	except:
		ArmaturePhysics.GUI_DICT={}
	
	try:
		dummy=ArmaturePhysics.GUI_DICT[ArmatureName]   	#Checking if previous GUI settings exist
	except:
		ArmaturePhysics.GUI_DICT[ArmatureName]={}
		ArmaturePhysics.GUI_DICT[ArmatureName]['stepsPerFrame']=MainParameterDict['stepsPerFrame']
		ArmaturePhysics.GUI_DICT[ArmatureName]['dynamicDerivative']=MainParameterDict['dynamicDerivative']
		ArmaturePhysics.GUI_DICT[ArmatureName]['boneforce']=MainParameterDict['boneforce']
		ArmaturePhysics.GUI_DICT[ArmatureName]['dynamicFlag']=MainParameterDict['dynamicFlag']
		ArmaturePhysics.GUI_DICT[ArmatureName]['objectForce']=MainParameterDict['objectForce']
		ArmaturePhysics.GUI_DICT[ArmatureName]['gravity']=MainParameterDict['gravity']
		ArmaturePhysics.GUI_DICT[ArmatureName]['dynamicNiteration']=MainParameterDict['dynamicNiteration']
		ArmaturePhysics.GUI_DICT[ArmatureName]['maxiter']=MainParameterDict['maxiter']
		ArmaturePhysics.GUI_DICT[ArmatureName]['dynamicFastFlag']=MainParameterDict['dynamicFastFlag']
		ArmaturePhysics.GUI_DICT[ArmatureName]['dampMode']=MainParameterDict['dampMode']
		
		ArmaturePhysics.GUI_DICT[ArmatureName]['almostRigidFlag']=MainParameterDict['almostRigidFlag']
		ArmaturePhysics.GUI_DICT[ArmatureName]['updateAlmostRigidFlag']=MainParameterDict['updateAlmostRigidFlag']
		
	########### Settings for frames
	
	try:
		StartVal=ArmaturePhysics.GUI_start   #Reads back previous GUI settings - if they exist
	except:
		StartVal=FrameStartEnd[0]
		
	try:
		SpanVal=ArmaturePhysics.GUI_span
	except:
		SpanVal=FrameStartEnd[1]-FrameStartEnd[0] 		#Only works in the GUI for positive spans.
		
	try:
		BackVal=ArmaturePhysics.GUI_back
	except:
		BackVal=0			#WOne ould automize this to be 1, if start frame smaller than end frame.
			
	start=Draw.Create(StartVal)		#I'm not sure what this does - but it helps keep the number value when you change window
	span=Draw.Create(SpanVal)
	back=Draw.Create(BackVal)
	
	########### Regular settings
	
	grav=Draw.Create(ArmaturePhysics.GUI_DICT[ArmatureName]['gravity'])		#I'm not sure what this does - but it helps keep the number value when you change window
	object=Draw.Create(ArmaturePhysics.GUI_DICT[ArmatureName]['objectForce'])
	dyn=Draw.Create(ArmaturePhysics.GUI_DICT[ArmatureName]['dynamicFlag'])
	boneforce=Draw.Create(ArmaturePhysics.GUI_DICT[ArmatureName]['boneforce'])
	steps=Draw.Create(ArmaturePhysics.GUI_DICT[ArmatureName]['stepsPerFrame'])
	dynderivative=Draw.Create(ArmaturePhysics.GUI_DICT[ArmatureName]['dynamicDerivative'])
	ndyn=Draw.Create(ArmaturePhysics.GUI_DICT[ArmatureName]['dynamicNiteration'])
	nmax=Draw.Create(ArmaturePhysics.GUI_DICT[ArmatureName]['maxiter'])
	dampmode=Draw.Create(ArmaturePhysics.GUI_DICT[ArmatureName]['dampMode'])
	dynfast=Draw.Create(ArmaturePhysics.GUI_DICT[ArmatureName]['dynamicFastFlag'])
	almostrigid=Draw.Create(ArmaturePhysics.GUI_DICT[ArmatureName]['almostRigidFlag'])
	updatealmostrigid=Draw.Create(ArmaturePhysics.GUI_DICT[ArmatureName]['updateAlmostRigidFlag'])


def draw1():
	#Setting up the buttons.
	
	global start,span,back
	global grav,object,dyn,boneforce,steps,dynderivative,ndyn,nmax,dampmode,dynfast,almostrigid,updatealmostrigid
	
	W=90	#Button width
	H=90   	#Button height
	D=10	#Distance between button rows	
	d=2
	
	S=(W*4)/10	
	
	F=(8*W)/10
	f=(2*W)/10
	
	Draw.Toggle(ArmatureName,				10,	0,				5*H,		3*W+D+S-f,20,0,			"Just displays the name of the armature")
	Draw.Toggle("EXIT",						4,	2*W+D+S-f,0*H,		W,H-2*d,0,					"Exits graphical user interface. Alternatively you could press 'Esc'")
	Draw.Toggle("MONITOR",					2,	0,				0*H,		W,H,0,  				"Monitors the total force acting on the object")	
	Draw.Toggle("EVOLVE",					1,	0,				1*H,		W,H,0,					"Evolves the object and the armature bones") 		# Draws a button. First number 1 means button nr 1.
	start=Draw.Number("Start",				3,	0,				2*H,		W,(H*2)/3,start.val,3,1000,	"Frame number to start evolution at") 
	span=Draw.Number("Span",				5,	0,				2*H+(H*2)/3,		W,(H*2)/3,span.val,1,1000,	"Number of frames to evolve") 	
	back=Draw.Toggle("Backwards",			6,	2*W+D+S-f,1*H,		W,H-2*d,back.val,			"Toggle evolve backwards")
	grav=Draw.Toggle("Gravity",				7,	1*W+D,1*H,		W,H-2*d,grav.val,			"Toggle gravity on/off")
	object=Draw.Toggle("Object",			8,	1*W+D,2*H,		W,H-d,object.val,			"Toggle object evolution on/off")
	dyn=Draw.Toggle("Dynamic",				9,	1*W+D,3*H+d,		W,H-d,dyn.val,			"Toggle dynamical bones evolution on/off")
	boneforce=Draw.Toggle("Bone force", 	11,	1*W+D,0*H,		W,H-2*d,boneforce.val,		"Toggle use of force bones on/off")
	steps=Draw.Number("Steps/Fr",			12,	0*W,2*H+2*(H*2)/3,		W,(H*2)/3,steps.val,1,10,		"Number of steps per time frame") 	
	dynderivative=Draw.Toggle("Deriv",		13,	3*W+D-f,2*H,		S,H*2,dynderivative.val,	"An old way of doing accurate (but slow) simultaneous object and dynamic evolution. Tip: Instead set dynAndObjectFlag=True in the InitialParameterDict. This is faster and more accurate.") 	
	ndyn=Draw.Number("Dyn",					14,	2*W+D,3*H+d,F,H/2-d,ndyn.val,1,5,		"Number of iterations for dynamic bones, per step") 	
	nmax=Draw.Number("Obj",					15,	2*W+D,2*H,		F,H-d,nmax.val,1,5,		"Maximum number of object iterations, per step") 	
	dynfast=Draw.Toggle("Dyn Fast",			16,	2*W+D,3*H+(3*H)/4,		F,H/4,dynfast.val,			"Speed up dynamic evolution. Only works accurately with free (non-constrained) dynamical bones") 	
	dampmode=Draw.Number("Damp",			17,	2*W+D,3*H+(2*H)/4,		F,H/4,dampmode.val,0,2,		"Damping mode. 0 is fast but can be unstable for large dampings. 1 is slow but accurate. 2 is slow and very stable.") 	
	almostrigid=Draw.Toggle("Almost-Rigid",	18,	1*W+D,4*H+2*d,		W,H-4*d,almostrigid.val,			"Speed up dynamic evolution. Good for trees for example.") 	
	updatealmostrigid=Draw.Toggle("Update A-Rigid",	19,	2*W+D,4*H+2*d,		W,H-4*d,updatealmostrigid.val,			"Speed up dynamic evolution. Good for trees for example.") 	
	
	
def event(evt,val):
	#Defining keyboard press events.
	
	if evt == Draw.ESCKEY: 			# If esc key pressed.
		Draw.Exit()    				# exit script
		return                 		# return from the function


def button(evt):
	#Defining what happens as one clicks the buttons
	
	if evt == 1: 	 		# If button 1 is pressed
		
		reload(ArmaturePhysics)							#For the development phase.
		endframe=start.val+span.val*(1-2*back.val)
		
		if endframe<1:
			Draw.PupMenu('Setting last frame of backwards evolution to 1')
			endframe=1
		
		MainParameterDict['FrameStartEnd']=[start.val,endframe]
		for name in ArmaturePhysics.GUI_DICT[ArmatureName]:
			MainParameterDict[name]=ArmaturePhysics.GUI_DICT[ArmatureName][name]
		
		ArmaturePhysics.MainEvolve(MainParameterDict,PhysOb)
			
	if evt == 2:
		MonitorParameterDict['gravity']=grav.val
		MonitorParameterDict['FrameStartEnd']=[start.val,start.val+span.val*(1-2*back.val)]
		MonitorParameterDict['boneforce']=boneforce.val
		ArmaturePhysics.Monitor(MonitorParameterDict,PhysOb)
			
	if evt == 3:
		ArmaturePhysics.GUI_start=start.val 	#This is just to store values of this session in module ArmaturePhysics
		
	if evt == 5:
		ArmaturePhysics.GUI_span=span.val 	    #This is just to store values of this session in module ArmaturePhysics
	
	if evt == 6:
		ArmaturePhysics.GUI_back=back.val 	    #This is just to store values of this session in module ArmaturePhysics
		
	if evt==4:				
		Draw.Exit()    		# Exit script
		return              # Return from the function
	
	if evt == 7:
		ArmaturePhysics.GUI_DICT[ArmatureName]['gravity']=grav.val 	    		#This is just to store values of this session in module ArmaturePhysics
				
	if evt == 8:
		ArmaturePhysics.GUI_DICT[ArmatureName]['objectForce']=object.val 	    #This is just to store values of this session in module ArmaturePhysics
		
	if evt == 9:
		ArmaturePhysics.GUI_DICT[ArmatureName]['dynamicFlag']=dyn.val
	
	if evt == 11:
		ArmaturePhysics.GUI_DICT[ArmatureName]['boneforce']=boneforce.val
	
	if evt == 12:
		ArmaturePhysics.GUI_DICT[ArmatureName]['stepsPerFrame']=steps.val
		
	if evt == 13:
		ArmaturePhysics.GUI_DICT[ArmatureName]['dynamicDerivative']=dynderivative.val
		
	if evt == 14:
		ArmaturePhysics.GUI_DICT[ArmatureName]['dynamicNiteration']=ndyn.val
	
	if evt == 15:
		ArmaturePhysics.GUI_DICT[ArmatureName]['maxiter']=nmax.val
	
	if evt == 16:			
		ArmaturePhysics.GUI_DICT[ArmatureName]['dynamicFastFlag']=dynfast.val
	
	if evt == 17:
		ArmaturePhysics.GUI_DICT[ArmatureName]['dampMode']=dampmode.val
	
	if evt == 18:
		ArmaturePhysics.GUI_DICT[ArmatureName]['almostRigidFlag']=almostrigid.val
	
	if evt == 19:
		ArmaturePhysics.GUI_DICT[ArmatureName]['updateAlmostRigidFlag']=updatealmostrigid.val
	
	
# Starting the graphical user interface 

def DrawGui(drawmode):
	#Starting up the graphical user interface.
	#The drawmode parameter is not used currently.
	
	Draw.Register(draw1,event,button)